#include <mygba.h>
#include "dma.h"
#include "ecran.h"
#include "touches.h"

//Dans le dossier Res: un tileset, une palette et une map!
#include "Res/Tower.til.c"
#include "Res/Tower.pal.c"
#include "Res/Tower.map.c"

//Copie un morceau de Tower_map sur ScreenBlock 31 avec l'offset dfini par defilX et defilY.
void CopieMap(int defilX, int defilY);

int main(void)         {
    int defilX, defilY;                         //Dfilement en X et en Y
    
    //On va utiliser le mode 0 et activer le BG0.
    REG_DISPCNT=1<<8;
    //Et maintenant, il faut paramtrer le BG0. J'utilise le mode palette 256 couleurs
    //(bit 7) et le screenblock 31 (bits 8  12).
    REG_BG0CNT=1<<7|31<<8;

    //Copie du tileset vers le charblock 0.
    CopieObjet(Tower_tiles,CharBaseBlock(0));
    //Copie la palette (Tower_palette) vers la palette physique.
    CopieObjet(Tower_palette,Palette);
    
    //Initialisation des variables
    defilX=defilY=0;                         //Initialise le dfilement au sommet  gauche
    

    //Boucle principale. Sur GBA, contrairement au PC, tous les programmes/jeux tournent
    //en boucle, et ne se terminent jamais. C'est le joueur qui choisira quand le terminer
    //en plaant l'interrupteur sur "Off". Mais l, on n'a pas  s'en soucier puisque
    //ce procd est mcanique et non logiciel :)
    while(1)        {
        //Dans un premier temps, je vais vrifier quelles touches sont appuyes et
        //dplacer l'cran en fonction de cela.

        /*
            1) Test des touches
            --------------------
            REG_TOUCHES contient l'tat des touches (1 si presses, 0 sinon). Pour tester
            l'tat d'une seule touche, il faut effectuer un ET bit  bit qui mettra  zro
            les bits qui ne nous intressent pas. Par exemple:
            Etat de REG_TOUCHES: 11 1001 0001 1110
            Masque TOUCHE_START: 00 0000 0000 1000
            Rsultat d'un AND:   00 0000 0000 1000 -> START est press!
        */

        /*
            2) Dplacement  l'cran
            ------------------------
            Rappelez-vous que les coordonnes x,y sont dfinies ainsi:
            0 --------------- x ------------------> max
            |
            |
            |
            y
            |
            |
            |
            V
            max                                      x max et y max
            Ceci explique donc les modifications de defilX et defilY en fonction des touches
            appuyes.
            Les positions maximales et minimales sont galement vrifies pour viter de
            sortir de la map. Peut-tre arriverez-vous  dterminer la formule qui me
            permet de trouver ces valeurs?
        */

        if (REG_TOUCHES&TOUCHE_DROITE && defilX<(64-30)*8)
            defilX++;
        if (REG_TOUCHES&TOUCHE_GAUCHE && defilX>0)
            defilX--;
        if (REG_TOUCHES&TOUCHE_BAS && defilY<(32-20)*8)
            defilY++;
        if (REG_TOUCHES&TOUCHE_HAUT && defilY>0)
            defilY--;

        /*
            Copie d'un morceau de map
            -------------------------
            
            L'cran fait 240x160 pixels, soit 30x20 pixels. Mais on doit laisser une
            mosaque de marge en haut et en bas, car si l'cran est dfil, on va voir
            partiellement la mosaque d'en dessus, et partiellement celle d'en dessous.
            L'cran fera donc 31x21 mosaques  ce moment-l (dont deux qui ne seront pas
            compltes).
            Et comme la taille de la map est limite, on ne peut la copier compltement
            d'un coup. C'est pourquoi j'extrais un morceau de la grande map (64x32)
            correspondant  ce dont on a besoin sur l'cran (31x21).
        */
        
        CopieMap(defilX,defilY);

        /*
            Defilement
            ----------
            Le dfilement ncessaire est partiellement effectu par CopieMap. En effet,
            pour un offset de 16 pixels par exemple, CopieMap ne copiera qu' partir du
            bloc n 2 (un bloc faisant 8x8 pixels). Comme vous ne verrez pas les blocs 0
            et 1, mais le 2 au sommet de l'cran, cela vous donnera l'impression que
            l'cran a dfil de deux blocs, soit de 16 pixels).
            Mais le dfilement rsiduel (chaque mosaque faisant 8 pixels, on ne peut pas
            trouver d'unit plus basse que celle-ci lors de la copie du morceau de map)
            est effectu par le matriel, grce aux registres ci-dessous.
        */
        REG_BG0HOFS=defilX%8;
        REG_BG0VOFS=defilY%8;


        //Il faut toujours se synchroniser avec le balayage vertical, sinon le programme
        //ira beaucoup trop vite!
        AttendVSync();
    }
    
    return 0;
}


void CopieMap(int defilX, int defilY)         {
    int i,j;                                    //Index de boucle
    unsigned short (*Map)[32];                  //Situe sur un screenblock

    /*
        Le dfilement donn par defilX et defilY est dfini au pixel prs. Mais dans notre
        cas, il doit tre effectu  la mosaque prs. Le dfilement supplmentaire 
        l'intrieur des mosaques est fait par le matriel  l'intrieur de la boucle
        principale.
    */
    defilX/=8, defilY/=8;

    //On va copier la map sur le screenblock 31 puisque BG0CNT est configur ainsi.
    Map=ScreenBaseBlock(31);

    //Pour la copie, une boucle for fera l'affaire.
    for (j=0;j<21;j++)
        for (i=0;i<31;i++)
            Map[j][i]=Tower_map[defilY+j][defilX+i];
}


/* END OF FILE */

