#include "commun.h"

//Variables diverses pour le texte
const u32 *fonte, *masque;
const char *tailles;
u32 hauteur_texte, largeur_texte;
u32 *adresse, largeur, hauteur;
u32 y_ww;
char *ligne2;
ZONE_TEXTE *zone_texte_courante;

//Certaines des fonctions qui se trouvent ci-dessous ne servent  rien dans le programme mais il pourrait
//tre utile de vouloir les utiliser. Si une fonction est marque comme inutilisee, ce n'est pas qu'elle ne
//sert  rien, mais qu'elle n'est pas utilise dans ce projet.

/*
    Fonctions de dessin.
    
    Le principe des fonctions  accs aux pixels a toujours t un peu du bidouillage. Et bien sr, la GBA
    ne fait pas exception  la rgle, car dessiner les pixels un  un serait impensable avec un processeur
     16 MHz. On va donc se dbrouiller pour crire plusieurs pixels  la fois. Voici la mthode que j'ai
    utilis:
    
    Aprs avoir trouv l'adresse du motif  modifier...
    Au dpart, il faut passer un masque (AND). Les valeurs  zro effaceront les pixels qui devront tre
    modifis. Ensuite on pourra crire la nouvelle valeur souhaite  l'aide d'un OR sans modifier les
    pixels qui ne font pas partie du motif. C'est commun comme mthode et cela permet de modifier plusieurs
    pixels en un accs.

    Prenons le cas o l'on veut dessiner un motif de 8 pixels (4 bits chacun)  la position x de 1. L'offset
    en y ne change absolument rien.
    D'abord: l'application du masque. Les valeur  0xf reprsentent l o on crira un pixel. Ce sera
    invers par la suite, comme on le verra. On le fait d'abord dcaler de 1 (x%8). Il faut savoir que par
    rapport  notre reprsentation d'un nombre hexa 0x12345678 par exemple, les pixels affichs  l'cran
    seront en fait 87654321. Donc pour faire dcaler  droite, on fera en fait dcaler le nombre vers la
    gauche.
    Masque: 0xffffffff, avec dcalage (4 bits car 1 pixel d'offset): 0xfffffff0.
    Aprs inversion, on obtient 0x0000000f. Seul les 4 premiers bits ne seront pas modifis, soit le
    premier pixel, comme voulu (car on dessine  la position 1, donc pas de modification du pixel 0!).
    Le faire avec un masque standard n'et pas t possible. Car si on se retrouve avec 0x00000000
    (=effacer tous les pixels), un dcalage dans quelle direction que ce soit ne donnera pas 0x0000000f
    car les bits dcals sont remplis avec des zros.
    Maintenant l'criture se fait de la mme manire avec un OR. Un motif de 0x12345678 sera, aprs
    dcalage, crit 0x23456780, (les bits  l'extrmit ne seront pas crits. Le pixel 1 n'est
    dfinitivement pas modifi (ouf!).
    Ensuite, il faut crire la fin du motif (le pixel restant). Pour cela, on fait la mme chose mais
    avec le comble du dcalage (8-dcalage courant), donc un dcalage de 7 dans ce cas-l. Je vous laisse
    refaire le raisonnement avec cette valeur, on obtient bien ce qu'il faut...
*/

//Efface un pixel (Inutilise)
void IN_IWRAM ClrPix(u32 x, u32 y)       {
    u32 *adr;
    
    //Calcule l'adresse pour se trouver dans la bonne mosaque.
    adr=(adresse+(x&~7)+((y&~7)*largeur));
    //Y ajoute la ligne (modulo 8). Chaque ligne de 8 pixels fait 32 bits (4 bits par pixel).
    adr+=(y&7);
    //Stoque le pixel (prend un pixel de valeur 0xf et le fait dcaler de 4 bits pour x modulo 8).
    *adr&=~(0xf<<((x&7)<<2));
}

//Met un pixel  la valeur 0xf (couleur 15 de la palette). Inutilise.
void IN_IWRAM SetPix(u32 x, u32 y)       {
    u32 *adr;
    adr=(adresse+(x&~7)+((y&~7)*largeur));
    adr+=(y&7);

    *adr|=0xf<<((x&7)<<2);
}

//Ecrit un pixel d'une certaine couleur.
void IN_IWRAM EcritPixel(u32 x, u32 y, u32 couleur)       {
    u32 *adr;
    adr=(adresse+(x&~7)+((y&~7)*largeur));
    adr+=(y&7);

    //D'abord effacer le contenu (masque, ClrPix)
    *adr&=~(0xf<<((x&7)<<2));
    //Ensuite stoquer, non pas 0xf comme dans SetPix mais simplement la couleur dsire.
    *adr|=couleur<<((x&7)<<2);
}

/*
                                            PrepareMap
    Prpare un morceau de map pour qu'on puisse y crire du texte. Dans la zone en question, chaque mosaque
    aura une autre valeur, de sorte que chacune soit unique.

Paramtres:
    adresse: Pointeur sur la map  modifier. Cela peut tre un ScreenBaseBlock ou une map en cache.
    x et y: Positions de dpart ( la mosaque prs) o la modification va oprer.
    largeur et hauteur: Taille du rectangle modifi ( la mosaque prs).
*/
//L'offset est le numro de la premire mosaque (en haut  gauche).
void IN_IWRAM PrepareMap(u16 *adresse, u32 x, u32 y, u32 offset, u32 largeur, u32 hauteur)       {
    int i,j;
    //La mosaque zro n'est pas utilise et est considre comme vide!
    offset++;
    for (j=0;j<hauteur;j++)
        for (i=0;i<largeur;i++)
            *(adresse+(j+y)*32+i+x)=offset+largeur*j+i;
}


//Ecrit un motif avec un masque
void IN_IWRAM EcritMotifMasque(u32 x, u32 y, u32 motif, u32 masque)        {
    u32 *adr;
    u32 i;
    adr=(adresse+(x&~7)+((y&~7)*largeur));
    adr+=(y&7);

    //Calcul du dcalage  faire (x modulo largeur mosaque) * 4 bits
    i=(x&7)<<2;
    //Masque (cf plus haut)
    *adr&=~(masque<<i);
    //Stoquage avec un OR par-dessus.
    *adr|=motif<<i;
    
    //Entre deux mosaques? (Il n'y a qu' la position 0 que le motif est parfaitement align).
    if (x&7)        {
        //Il faut reporter la partie manquante du motif sur la mosaque  sa droite
        adr+=8;
        //Si on a par exemple dcal de 2 avant, ici on dcalera de 6.
        i=32-i;
        *adr&=~(masque>>i);
        *adr|=motif>>i;
    }
}

//Ecrit plusieurs motifs et plusieurs masques
void IN_IWRAM EcritMotifsMasque(u32 x, u32 y, u32 nombre, const u32 *motifs, const u32 *masques)        {
    u32 *adr;
    u32 i,j;
//    adr=((u32*)CharBaseBlock(2)+((x>>3)<<3)+((y>>3)<<8));
    adr=(adresse+(x&~7)+((y&~7)*largeur));
    adr+=(y&7);

    //Prparation
    i=(x&7)<<2;
    if (x&7)        {
        j=32-i;
        while(nombre--)       {
            *adr&=~(*masques<<i);
            *adr|=*motifs<<i;
            *(adr+8)&=~(*masques>>j);
            *(adr+8)|=*motifs>>j;
            if ((y&7)==7)
                adr+=(largeur<<3)-7;
            else
                adr++;
            masques++, motifs++;
            y++;
        }
    }
    else    {
        while(nombre--)       {
            *adr&=~(*masques<<i);
            *adr|=*motifs<<i;
            if ((y&7)==7)
                adr+=(largeur<<3)-7;
            else
                adr++;
            masques++, motifs++;
            y++;
        }
    }
}

//Ecrit plusieurs motifs, plusieurs masques et effectue une multiplication.
void IN_IWRAM EcritMotifsMasqueCouleur(u32 x, u32 y, u32 nombre, const u32 *motifs, const u32 *masques, u32 couleur)        {
    u32 *adr;
    u32 i,j;
//    adr=((u32*)CharBaseBlock(2)+((x>>3)<<3)+((y>>3)<<8));
    adr=(adresse+(x&~7)+((y&~7)*largeur));
    adr+=(y&7);

    //Prparation
    i=(x&7)<<2;
    if (x&7)        {
        j=32-i;
        while(nombre--)       {
            *adr&=~(*masques<<i);
            *adr|=(*motifs*couleur)<<i;
            *(adr+8)&=~(*masques>>j);
            *(adr+8)|=(*motifs*couleur)>>j;
            if ((y&7)==7)
                adr+=(largeur<<3)-7;
            else
                adr++;
            masques++, motifs++;
            y++;
        }
    }
    else    {
        while(nombre--)       {
            *adr&=~(*masques<<i);
            *adr|=(*motifs*couleur)<<i;
            if ((y&7)==7)
                adr+=(largeur<<3)-7;
            else
                adr++;
            masques++, motifs++;
            y++;
        }
    }
}

//Dessine une chane de caractres aux positions demandes (Mode 0)
void IN_IWRAM DessTexte(u32 x, u32 y, char *chaine, u32 couleur)     {
    while(*chaine)      {
        if (x+tailles[*chaine-32]>largeur<<3)
            break;
        EcritMotifsMasqueCouleur(x,y,hauteur_texte,fonte+hauteur_texte*(*chaine-32),masque+hauteur_texte*(*chaine-32),couleur);
        x+=tailles[*chaine-32];
        chaine++;
    }
}

//Dessine un caractre (Mode 0)
void DessCar(u32 x, u32 y, u8 car, u32 couleur)     {
    Exec(EcritMotifsMasqueCouleur,x,y,hauteur_texte,fonte+hauteur_texte*(car-32),masque+hauteur_texte*(car-32),couleur);
}

//Idem, en italique.
void DessCarItalique(u32 x, u32 y, u8 car, u32 couleur)     {
    u32 i;
    for (i=0;i<hauteur_texte/3;i++)
        Exec(EcritMotifMasque,x+2,y+i,fonte[hauteur_texte*(car-32)+i]*couleur,masque[hauteur_texte*(car-32)+i]);
    for (;i<2*hauteur_texte/3;i++)
        Exec(EcritMotifMasque,x+1,y+i,fonte[hauteur_texte*(car-32)+i]*couleur,masque[hauteur_texte*(car-32)+i]);
    for (;i<hauteur_texte;i++)
        Exec(EcritMotifMasque,x+0,y+i,fonte[hauteur_texte*(car-32)+i]*couleur,masque[hauteur_texte*(car-32)+i]);
}

//Dessine du texte avec un support formatage basique et avec des retours  la ligne automatiques.
void IN_IWRAM DessTexteWW(u32 x, u32 y, char *chaine, u32 couleur)     {
    u32 xx, xbase=x, ybase=y, gras=0;
    char *temp;

    while(*chaine)      {
        if (*chaine==32)     {
            temp=chaine+1;
            xx=x+tailles[0];
            while (*temp!=32 && *temp!='\n' && *temp)      {
                xx+=tailles[*temp-32];
                temp++;
            }
            if (xx>=largeur<<3)     {
haut:
                if (y==ybase)
                    ligne2=chaine+1;
                y+=hauteur_texte, x=xbase;
                if (y+hauteur_texte>hauteur<<3)
                    break;
                chaine++;
                continue;
            }
        }
        if (*chaine=='')         {
            chaine++;
            if (*chaine=='*')
                gras=!gras;
            else
                couleur=(*chaine)-'0';
        }
        else    {
            if (*chaine=='\n')
                goto haut;
            if (x+tailles[*chaine-32]>largeur<<3)
                goto haut;
            EcritMotifsMasqueCouleur(x,y,hauteur_texte,fonte+hauteur_texte*(*chaine-32),masque+hauteur_texte*(*chaine-32),couleur);
            if (gras)
                EcritMotifsMasqueCouleur(x+1,y,hauteur_texte,fonte+hauteur_texte*(*chaine-32),masque+hauteur_texte*(*chaine-32),couleur);
            x+=tailles[*chaine-32];
        }
        chaine++;
    }
    y_ww=y;
}

//Inutilise. Peut tre utile de l'avoir.
/*void DessTexteWWAvecScrolling(u32 x, u32 y, u32 scroll_y, char *chaine, u32 couleur)     {
    u32 xx, xbase=x, ybase=y;
    char *temp;

    while(*chaine)      {
//        EcritMotifsMasque(x,y,12,Verdana_fonte+12*(*chaine-32),Verdana_masque+12*(*chaine-32));
        if (*chaine==32)     {
            temp=chaine+1;
            xx=x+tailles[0];
            while (*temp!=32 && *temp!='\n' && *temp)      {
                xx+=tailles[*temp-32];
                temp++;
            }
            if (xx>=largeur<<3)     {
haut:
                y+=hauteur_texte, x=xbase;
                if (y+hauteur_texte+ybase>(hauteur<<3)+scroll_y)
                    break;
                chaine++;
                continue;
            }
        }
        if (*chaine=='\n')
            goto haut;
        if (x+tailles[*chaine-32]>largeur<<3)
            goto haut;
        if (y-ybase>=scroll_y)
            Exec(EcritMotifsMasqueCouleur,x,y-scroll_y+ybase,hauteur_texte,fonte+hauteur_texte*(*chaine-32),masque+hauteur_texte*(*chaine-32),couleur);
        x+=tailles[*chaine-32];
        chaine++;
    }
    y_ww=y;
}*/

//Change la palette (sur seize) utilise dans une certaine zone d'une map.
void ChangePaletteMap(u16 *adresse, u32 x, u32 y, u32 largeur, u32 hauteur, u32 nvPalette)       {
    int i,j;
    //La palette est dfinie dans les 4 derniers bits d'une mosaque. On dcale donc le numro de palette par douze.
    for (j=0;j<hauteur;j++)
        for (i=0;i<largeur;i++)
            *(adresse+(j+y)*32+i+x)|=nvPalette<<12;
}

//Slectionne un zone de texte.
void SelectZoneTexte(ZONE_TEXTE *z)      {
    adresse=(u32*)z->adresse;
    largeur=z->largeur, hauteur=z->hauteur;
    zone_texte_courante=z;
}

//Efface une zone de texte
void EffaceZone(ZONE_TEXTE *z)      {
    u32 zero=0;
    MemSet((void*)&zero,(void*)z->adresse,z->largeur*z->hauteur*8,DMA_32im);
}

/*
    Support du mode 0 en 256 couleurs. Pas utilis, mais pourrait tre utile...
*/
u32 *adresse256;
u32 largeur256, hauteur256;

//La meilleure pour le moment
void IN_IWRAM EcritPixel256(u32 x, u32 y, u32 couleur)       {
    u32 *adr;
    adr=adresse256+(((x&~7)+((y&~7)*largeur))<<1);
    adr+=(y&7)<<1;
    adr+=(x&7)>>2;

    *adr&=~(0xff<<((x&3)<<3));
    *adr|=couleur<<((x&3)<<3);
}

//Ecrit un motif avec un masque
void IN_IWRAM EcritPlusDe4Pixels256(u32 x, u32 y, u32 nbre, u32 *motif)        {
    u32 *adr;
    u32 i;
    const int masque=0xffffffff;

    adr=adresse256+(((x&~7)+((y&~7)*largeur))<<1);
    adr+=(y&7)<<1;
    adr+=(x&7)>>2;

    while(nbre--)     {
        i=(x&3)<<3;

        *adr&=~(masque<<i);
        *adr|=(*motif)<<i;

        if ((x&7)>=4)
            adr+=15;
        else
            adr++;

        i=(4-(x&3))<<3;
        *adr&=~(masque>>i);
        *adr|=(*motif)>>i;

        motif++;
        x+=4;
    }
}

/*
    Mode 4: Pour le lecteur de textes.
    Comme la fonction de ces fonctions est identique  celles pour le mode 0, je me passerai de commentaires.
*/
void IN_IWRAM Mode4_EcritPlusDe4Pixels(u32 x, u32 y, u32 nbre, u32 *motif)        {
    u32 *adr;
    u32 i,j;
    const int masque=0xffffffff;

    adr=(u32*)MemVideo+60*y+(x>>2);

    j=(x&3)<<3;

    while(nbre--)     {
        i=j;

        *adr&=~(masque<<i);
        *adr|=(*motif)<<i;

        adr++;

        i=32-j;
        //Ce n'est pas une erreur (<<)
        *adr&=~(masque<<i);
        *adr|=(*motif)>>i;

        motif++;
    }
}

void IN_IWRAM Mode4_EcritPlusDe4PixelsMasque(u32 x, u32 y, u32 nbre, u32 *motif, u32 *masque)        {
    u32 *adr;
    u32 i,j;

    adr=(u32*)MemVideo+60*y+(x>>2);

    j=(x&3)<<3;

    while(nbre--)     {
        i=j;

        *adr&=~((*masque)<<i);
        *adr|=(*motif)<<i;

        adr++;

        i=32-j;
        *adr&=~((*masque)>>i);
        *adr|=(*motif)>>i;

        motif++, masque++;
    }
}

void IN_IWRAM Mode4_EcritPlusDe4PixelsMasqueCouleur(u32 x, u32 y, u32 nbre, u32 *motif, u32 *masque, u32 couleur)        {
    u32 *adr;
    u32 i,j,val;

    adr=(u32*)MemVideo+60*y+(x>>2);

    j=(x&3)<<3;

    while(nbre--)     {
        val=(*motif)*couleur;

        i=j;
        *adr&=~((*masque)<<i);
        *adr|=val<<i;

        adr++;

        i=32-j;
        *adr&=~((*masque)>>i);
        *adr|=val>>i;

        masque++, motif++;
    }
}

void IN_IWRAM Mode4_EcritPixel(u32 x, u32 y, u32 couleur)       {
    u32 *adr;
    adr=(u32*)MemVideo+60*y+(x>>2);

    *adr&=~(0xff<<((x&3)<<3));
    *adr|=couleur<<((x&3)<<3);
}


//Pour le mode 4. Supporte le clipping (pas de dpassement en haut ou en bas de l'cran, mais attention: pas horizontalement).
void IN_IWRAM DessCar256(u32 x, s32 y, u8 car, u32 couleur)     {
    int i=0,h=hauteur_texte;
    u32 o;

    //Clipping (en y)
    if (y<0)        {
        if (y+h<0)
            return;
        i+=-y;
    }
    if (y+h>=160)       {
        if (y>=160)
            return;
        h-=(y+h)-160;
    }

    o=(hauteur_texte*largeur_texte)*(car-32)+i*largeur_texte;
    for (;i<h;i++)      {
        Mode4_EcritPlusDe4PixelsMasqueCouleur(x,y+i,largeur_texte,(u32*)(fonte+o),(u32*)(masque+o),couleur);
        o+=largeur_texte;
    }
}

//Idem.
void IN_IWRAM DessCarItalique256(u32 x, s32 y, u8 car, u32 couleur)     {
    int i=0,h=hauteur_texte;
    u32 o;

    //Clipping (en y)
    if (y<0)        {
        if (y+h<0)      {
            return;
        }
        i+=-y;
    }
    if (y+h>=160)       {
        if (y>=160)
            return;
        h-=(y+h)-160;
    }

    o=(hauteur_texte*largeur_texte)*(car-32)+i*largeur_texte;
    for (;i<h;i++)          {
        Mode4_EcritPlusDe4PixelsMasqueCouleur(x+(2-i/3),y+i,largeur_texte,(u32*)(fonte+o),(u32*)(masque+o),couleur);
        o+=largeur_texte;
    }
}


//Fonction de dbogage. A dcommenter si vous voulez l'utiliser en mode 0 (pendant la map).
/*void Debug(const char *format, ...)	{
    int j;
	char      str[100];
	va_list   arg_ptr;
	ZONE_TEXTE *z=zone_texte_courante;

    ChangeFonte(Verdana);
    SelectZoneTexte(&zt_debug);
    EffaceZone(&zt_debug);

	va_start(arg_ptr, format);
	j=vsprintf(str, format, arg_ptr);

    Exec(DessTexte,0,0,str,1);
    CopieDMA((void*)buffer_debug,(void*)CharBaseBlock(3)+32,sizeof(buffer_debug)>>1,DMA_16im);

	while(!(REG_TOUCHES&TOUCHE_SELECT));
	while((REG_TOUCHES&TOUCHE_SELECT));
	
	SelectZoneTexte(z);
}*/

//Fonction de dbogage. A dcommenter si vous voulez l'utiliser en mode 4 (pendant le lecteur de textes).
void Debug(const char *format, ...)	{
    int j=0,x=0;
	char      str[100], *ptr=str;
	va_list   arg_ptr;

    ChangeFonte256(Verdana);
    MemSet(&j,MemVideo,120*12,DMA_16im);

	va_start(arg_ptr, format);
	j=vsprintf(str, format, arg_ptr);

    while(*ptr)     {
        Exec(DessCar256,x,0,*ptr,0xfe);
        x+=tailles[*ptr-32];
        ptr++;
    }

	while(!(REG_TOUCHES&TOUCHE_SELECT));
	while((REG_TOUCHES&TOUCHE_SELECT));
}

//Variables globales pour rcuprer le travail de DessTexteSpecial
LIEN IN_EWRAM liens[30];
u32 nEvenement;
char IN_EWRAM message_fond[100];

//Utilise pour dessiner un menu. Support un offset vertical (scrolling en y), un handler auquel il demande
//les informations ncessaires, et met en couleur diffrente la slection active.
//Remplit les liens trouves dans les balises et le message associ  la slection courante.
void IN_IWRAM DessTexteSpecial(u32 x, u32 y, char *chaine, u32 couleur, u32 selection, u32 offset, unsigned char *(*handler)(), u8 bAffiche)     {
    u32 xx, xbase=x, coul=couleur, interieur=0;
    char *temp, *chnesauve=NULL;
    u32 i, j;

    //A chaque mouvement, le menu est entirement ranalys et raffich. Tous les vnements prcdemment
    //trouvs sont vids puis recrs. Mme le texte qui n'est pas visible dans le menu est analys pour
    //en connatre la largeur. Il ne sera seulement pas affich dans ce cas-l.
    nEvenement=0;                               //Au dpart, il n'y a aucun vnement

redo:
    //Tant qu'on ne rencontre pas de caractre nul (fin de chane)
    while(*chaine)      {
        //C'est un espace
        if (*chaine==32)     {
            //On saute l'espace courant. Ce pointeur sera utilis pour trouver le prochain espace.
            temp=chaine+1;
            //On va tester la taille de texte qu'il y a jusqu'au prochain espace pour savoir si cela ferait dborder la ligne.
            xx=x+tailles[0];
            //Tant que ce n'est pas un espace ou un saut de ligne et que la chane n'est pas termine...
            while (*temp!=32 && *temp!='\n' && *temp!='#' && *temp)      {
                //On ajoute la taille
                xx+=tailles[*temp-32];
                //On incrmente le pointeur
                temp++;
            }
            //A-t-on dpass la largeur maximale? (la largeur est dfinie en nombre de tiles, il faut donc la
            //multiplier par huit puisqu'on travaille en pixels, ce qui reprsente un dcalage de 3 vers la gauche).
            if (xx>=largeur<<3)     {
haut:                                           //Label appeler pour passer  la prochaine ligne
                //Prochaine ligne
                y+=hauteur_texte;
                //A l'intrieur d'une option de menu?
                if (interieur)
                    x=xbase+9;                      //Oui -> indentation (9 pixels)
                else
                    x=xbase;                        //Non -> Reprise  gauche
                //Prochain caractre.
                chaine++;
                //Reprise au dbut de la boucle (le caractre n'est plus valide  prsent).
                continue;
            }
        }

        //Caractre dise: Insre un vnement si le tableau n'est pas dj rempli
        if (*chaine=='#' && nEvenement<sizeof(liens)/sizeof(liens[0])-1)           {
            //Prochain caractre
            chaine++;
            //L'vnement en question est slectionn -> couleur+1
            if (selection==nEvenement)
                coul++;
            //On va stocker le texte du lien dans le champ 'symbole' du lien
            i=0;
            //Tant que le champ n'est pas rempli, que ce n'est pas la fin du lien, de la chane ou enfin le dbut du message
            while(*chaine!='#' && *chaine!='|' && *chaine && i<sizeof(liens[nEvenement].symbole)-1)
                liens[nEvenement].symbole[i++]=*chaine++;
            //Toujours terminer la chane ;-)
            liens[nEvenement].symbole[i]=0;
            //Stocke la longueur dans j
            j=i;
            //On recommence
            i=0;
            //Dans le cas o l'option n'est pas slectionn, le texte du message ne nous importe pas.
            if (selection==nEvenement)          {
                //Un message est dfini
                if (*chaine=='|')
                    chaine++;
                //Jusqu' la fermeture de la balise du lien
                while(*chaine!='#' && *chaine && i<sizeof(message_fond)-1)
                    message_fond[i++]=*chaine++;
                message_fond[i]=0;
            }
            else    {
                //Jusqu' la fermeture du lien
                while(*chaine!='#' && *chaine)
                    chaine++;
            }
            //Position verticale
            liens[nEvenement].y=y;
            //Prochain caractre (passe le dise)
            chaine++;
            //Texte du lien nul -> on questionne le handler
            if ((*chaine==0 || *chaine=='\n') && handler)          {
                liens[nEvenement].demande=nEvenement;
                chnesauve=chaine;
                chaine=handler(DEM_OPTION,nEvenement,0);
                if (chaine==NULL)
                    chaine=chnesauve, chnesauve=NULL;
            }
            else
                liens[nEvenement].demande=-1;
            nEvenement++;
            if (x==xbase)
                x+=9;
            interieur=1;
            continue;
        }
        else if (*chaine=='\n')      {
            if (interieur)      {
                liens[nEvenement-1].yfin=y+hauteur_texte;
                coul=couleur;
            }
            interieur=0;
            goto haut;
        }
        if (x+tailles[*chaine-32]>largeur<<3)
            goto haut;
        if (bAffiche && (signed)y-(signed)offset>=12 && (signed)y-(signed)offset+hauteur_texte<64)
            EcritMotifsMasqueCouleur(x,y-offset,hauteur_texte,fonte+hauteur_texte*(*chaine-32),masque+hauteur_texte*(*chaine-32),coul);
        x+=tailles[*chaine-32];
        chaine++;
    }
    
    if (chnesauve)      {
        chaine=chnesauve;
        chnesauve=NULL;
        goto redo;
    }

/*    if (x==xbase)
        y-=hauteur_texte;*/

    if (interieur)
        liens[nEvenement-1].yfin=y+hauteur_texte;
    y_ww=y;
}

void IN_IWRAM DessTexte256(u32 x, u32 y, char *chaine, u32 couleur)     {
    while(*chaine)      {
        if (x+tailles[*chaine-32]>240)
            break;
        DessCar256(x,y,*chaine,couleur);
        x+=tailles[*chaine-32];
        chaine++;
    }
}


