//Un tableau en cache utilis pour la map. On ne peut pas directement accder  l'cran car s'il est en
//train d'tre redessin  ce moment, le rsultat ne sera pas trs beau car une partie de l'affichage
//utilisera les premiers paramtres et une autre les nouveaux. C'est pourquoi on effectue toutes les
//copies durant une priode appele VBLANK, o le contrleur graphique "remonte" de la dernire ligne de
//l'affichage jusqu' la premire. La VBLANK fait dure environ 68 lignes de LCD. Lorsqu'on effectue des
//des oprations durant les 160 lignes (hauteur du LCD) restantes, on est donc oblig de les mettre en
//cache d'abord, avant de copier le rsultat  la fin pendant la VBLANK.
unsigned short IN_EWRAM tableau[22][32];

/**********************************************************************************************************
                                        Moteur de gestion de mosaques
    Le problme est la limitation de la mmoire vido. Normalement on est cens charger toutes les
    mosaques qu'on serait susceptible d'utiliser d'abord en VRAM (mmoire vido) et ensuite les utiliser
    directement.
    Mais faisons le calcul. Notre map (256 couleurs, 8 bits), pour chaque mosaque (8x8 pixels), utilise
    64 octets. Or, il y a 64 ko de VRAM pour les arrire-plans, donc 1024 mosaques si la mmoire vido
    n'tait utilise que pour cela. Mais il faut encore compter qu'une partie sera utilise pour stocker
    les maps.
    Et notre map, aussi petite qu'elle soit, utilise dj 823 tiles (mosaques). En
    imaginant une application utilisant une carte un peu plus complique, on voit que la limite est vite
    atteinte.

    Ce moteur de tiles volu permet d'afficher sur une mme carte un nombre "indfini" de tiles dans une
    map, et sur un nombre "indfini" de plans diffrents.
    Forcment, la premire ide qui vient  l'esprit serait de charger dynamiquement les tiles dont on a
    besoin. Cela impliquerait qu'on associe  chaque tile sur l'cran un emplacement diffrent en VRAM.
    Il s'agit bien de la mthode utilise par les programmeurs commerciaux mais cela consomme normment
    de mmoire, et si on veut couvrir un seul plan entier en mode 256 couleurs (comme dans le cas du
    mien), il faudrait 31x21 tiles = 651 tiles, soit 65% de la mmoire.
    Ma mthode consiste  associer les tiles virtuelles (pas en VRAM et pas limites en nombre) aux tiles
    relles (qui se trouvent en VRAM). Chaque tile est analyse, puis on regarde si elle se trouve dj
    en VRAM. Si ce n'est pas le cas on la charge, sinon on l'utilise directement. Ainsi les tiles
    identiques peuvent tout de mme tre rutilises, conomisant de la mmoire vido. Les tiles sont
    entasses  la demande jusqu' saturation de la VRAM (nombre de tiles en VRAM > MAX_TILES dcid).
    A ce moment-l, il faut rechercher les tiles qui ont t charges mais qui ont disparues entre-temps
    pour pouvoir les rutiliser. C'est assez simple, un parcours de la map et une re-validation des tiles
    qui s'y trouvent encore permet de librer toutes celles qui n'ont pas t signales "prsentes".

    Ma mthode utilise moins de mmoire, et dpend de MAX_TILES qui est arbitraire (enfin rgl de manire
     tre sr qu'il n'y aura jamais plus de tiles que a sur le mme cran). Avec 512 tiles, cela
    reprsente un peu plus de 50%, mais on pourrait le rduire  256 par exemple, 512 c'tait juste pour
    prvoir large. De plus, elle fonctionne sur un nombre indfini de plans. En contrepartie, elle
    consomme pas mal de CPU en plus.
**********************************************************************************************************/

#define MAX_TILES 512                       //Nombre de tiles 8x8 maximal affichable en mme temps
#define MAX_PHYSICAL_TILES 4096             //Nombre maximum possible de tiles dans la map (il faut bien fixer une valeur)
unsigned short IN_EWRAM utilTiles[MAX_PHYSICAL_TILES];
unsigned short IN_EWRAM numTiles[2000];      //C'est trop. MAX_TILES+16*11 suffirait largement, je suppose.
unsigned short IN_EWRAM copieTiles[MAX_PHYSICAL_TILES];
unsigned short n_tiles;                     //Nombre de tiles utilises
#define TAILLEX_ECRAN 31                    //Nombre de mosaques  remplir sur l'cran
#define TAILLEY_ECRAN 21                    //Horizontalement et verticalement
int gblDepX, gblDepY;

//Un cran, ne ncessitant pas de contenir de buffer, mais juste ses coordonnes (dfilement en x,y)
typedef struct      {
    int x,y;
} ECRAN;

//Initialise les variables globales pour le systme de gestion des mosaques en VRAM.
void InitTileSystem()       {
    memset(utilTiles,0,sizeof(utilTiles));
    memset(numTiles,0,sizeof(numTiles));
    memset(copieTiles,0,sizeof(copieTiles));
    n_tiles=1;
}

//Initialise les coordonnes d'une structure ECRAN.
void InitEcran(ECRAN *ecr)      {
    ecr->x=ecr->y=0;
}

int depassement=0;                       //Uniquement pour la dtection d'erreurs. Il faut que l'on sache que si le jeu plante, il faudra prvoir plus de tiles  disposition ou (en l'occurence) rduire le nombre de tiles utilises par un monde. En imaginant que chaque tile est diffrente, cela ncessiterait 176 tiles, mais avec 128 tiles, je remplis dj la VRAM  moiti, reste le fond, le texte, ... faut pas exagrer!

//Copie la tile virtuelle sur la tile physique
void CopieTile(unsigned short tileVirtuelle, unsigned short tile)       {
    if (tile<MAX_TILES && tile>0)                       //Tests facultatifs, mais l pour viter qu'en cas de dbordement de tile le dsastre soit trop grand...
        CopieDMA((void*)plan_tiles+tileVirtuelle*64, (void*)CharBaseBlock(0)+64*tile, 32, DMA_16im);
}

//Optimise les tiles (enlves celles qui ne sont plus utilises)
void OptTiles()     {
    unsigned int i,j;
    unsigned short val;
        /****************************************************************************
            Vidage et optimisation de la VRAM:
            Cette partie va dterminer quelles tiles doivent tres retires de la
            mmoire vido. L'algo est le suivant:
            -Recherche des tiles communes entre les tiles dj existantes et celles
             que l'on veut pour afficher l'image courante.
            -Les tiles qui n'existent pas dj se voient allouer un bloc en mmoire
             vido.
            -Les tiles pour lequel on a allou un bloc mais qui ne l'utilisent plus
             (notamment parce qu'elles ne sont plus visible  l'cran) sont libres
             et pourront ensuite tre remplaces par les nouvelles tiles.
        ****************************************************************************/

        if (depassement)                //Dj eu lieu
            return;

        n_tiles=1;                      //Il n'y a plus qu'une seule tile  prsent
        depassement=1;

        //"Dvalide" les tiles utilises la dernire fois. On va les "revalider" ensuite. Si certaines d'entre elles n'est pas valide, alors elles seront jartes de la mmoire vido.
        for (i=1;i<MAX_TILES;i++)     {
            copieTiles[numTiles[i]]=0;
        }

        //Etape de validation
        for (j=0;j<TAILLEY_ECRAN;j++)     {
            for (i=0;i<TAILLEX_ECRAN;i++)     {
                val=plan_map[j+gblDepY][i+gblDepX]&2047;        //&2047 car les bits qui suivent sont des flags qui n'ont rien  faire l-dedans
                copieTiles[val]=1;              //La tile existe?
            }
        }

        //Ici, on va vrifier si chacune des tiles numres plus haut existe bien sur l'cran courant. Si ce n'est pas le cas, elle sera sortie de la VRAM.
        for (i=1;i<MAX_TILES;i++)     {
            if (copieTiles[numTiles[i]]==0)     {
                utilTiles[numTiles[i]]=0;                   //Relation entre tile virtuelle et relle -> fichue
                numTiles[i]=0;                              //Libre
            }
        }
}

//Recherche une tile relle disponible et y copie la tile virtuelle "val"
int IN_IWRAM ChercheTile(int val)      {
    while(numTiles[n_tiles]!=0)
        n_tiles++;

    //Pas que pour le dbogage!
    if (n_tiles>=MAX_TILES)        {
        //Pour le dbogage
        if (depassement)       {                            //Pour le dbogage seulement: dtection de boucle infinie (un dpassement a dj eu lieu)
//                Alerte("Max de tiles");
            return 0;
        }
        return 1;
    }

    //Stocke les rfrences entre les deux
    utilTiles[val]=n_tiles;
    numTiles[n_tiles]=val;

    //Copie la mosaque
    Exec(CopieTile,val,n_tiles);
    n_tiles++;
    return 0;
}

//Copie un bloc de 16x16 pixels sur une map de 8x8
void IN_IWRAM CopieBloc_8(int x, int y, unsigned short bloc)      {
    register unsigned short val=bloc;                   //Pittre optimiser a...

    if (bloc==0)                        //Optimisation pour bloc nul (attention, il doit pour cela tre toujours vide, vrifiez le tileset gnr!)
        goto ici;

rechercheTile:
    if (n_tiles>=MAX_TILES)         {       //Trop de tiles -> recours  l'optimisation
        Exec(OptTiles);
        val=bloc;
    }

    if (utilTiles[val]==0)     {            //Existe pas encore?
        if (ChercheTile(val))               //On la met et on revrifie si une optimisation est ncessaire
            goto rechercheTile;
    }

    bloc=utilTiles[val];                    //C'est bon, on a le numro de notre tile physique correspondante.

ici:
    //Stockage dans le tableau qui sera copi  la VBLANK.
    tableau[y][x]=bloc;
}

//Copie un morceau de map en fonction des paramtres. Reste  dplacer a dans la mmoire vido  la fin
void IN_IWRAM CopieMap(int departX, int departY,/* int tailleX, int tailleY,*/ int tailleMapX, int tailleMapY)        {
    unsigned int x,y;
    unsigned int i;
    unsigned short *ptr;
    const int tailleX=31, tailleY=21;                       //Taille de l'cran + 1 pour le dfilement ventuel

    gblDepX=departX, gblDepY=departY;
	for (y=0;y<tailleY;y++)        {
    	ptr=(unsigned short *)plan_map[y+departY]+departX;
        for (x=0;x<tailleX;x++)     {
            i=*ptr++;
            CopieBloc_8(x,y,i&2047);
        }
    }
}

//Efface un screenblock entier (2048 octets). Les screenblocks contiennent les donnes des maps.
void EffaceScreenBlock(u32 screenBlock)     {
    int zero=0;
    MemSet(&zero,ScreenBaseBlock(screenBlock),(0x0800)>>2,DMA_32im);
}






